@@ -35,78 +35,30 @@ from rest_framework.generics import (CreateAPIView,
     GenericAPIView, RetrieveAPIView)
 from rest_framework.mixins import CreateModelMixin
 from rest_framework.response import Response
-from rest_framework import serializers, status
+from rest_framework import status
 
 from ..backends import ProcessorError
 from ..compat import is_authenticated, StringIO
 from ..docs import swagger_auto_schema, OpenAPIResponse
 from ..mixins import BalanceAndCartMixin, CartMixin, InvoicablesMixin
 from ..models import CartItem
-from .serializers import (ChargeSerializer, InvoicableSerializer,
-    NoModelSerializer, PlanRelatedField, ProcessorAuthSerializer,
-    ValidationErrorSerializer)
+from .serializers import (CartItemCreateSerializer, CartItemUploadSerializer,
+    ChargeSerializer, CheckoutSerializer, OrganizationCartSerializer,
+    RedeemCouponSerializer, ValidationErrorSerializer)
 
 #pylint: disable=no-init
 LOGGER = logging.getLogger(__name__)
 
 
-class CartItemCreateSerializer(serializers.ModelSerializer):
-    """
-    Serializer to build a request.user set of plans to subscribe to (i.e. cart).
-    """
-    plan = PlanRelatedField(read_only=False, required=True,
-        help_text=_("The plan to add into the request.user cart."))
-
-    class Meta:
-        model = CartItem
-        fields = ('plan', 'option', 'full_name', 'sync_on', 'email')
-
-
-class OrganizationCartSerializer(NoModelSerializer):
-    """
-    Items which will be charged on an order checkout action.
-    """
-    processor = ProcessorAuthSerializer(required=False,
-      help_text=_("Keys to authenticate the client with the payment processor"))
-    results = InvoicableSerializer(many=True)
-
-
-class CheckoutItemSerializer(NoModelSerializer):
-    option = serializers.IntegerField(
-        help_text=_("selected plan option during checkout"))
-
-
-class CheckoutSerializer(NoModelSerializer):
-    """
-    Processor token to charge the cart items.
-    """
-    items = CheckoutItemSerializer(required=False, many=True)
-    remember_card = serializers.BooleanField(required=False,
-        help_text=_("attaches the payment card to the Organization when true"))
-    processor_token = serializers.CharField(required=False, max_length=255,
-        help_text=_("one-time token generated by the processor"\
-            "from the payment card."))
-    street_address = serializers.CharField(required=False, allow_blank=True,
-        help_text=_("Street address"))
-    locality = serializers.CharField(required=False, allow_blank=True,
-        help_text=_("City/Town"))
-    region = serializers.CharField(required=False, allow_blank=True,
-        help_text=_("State/Province/County"))
-    postal_code = serializers.CharField(required=False, allow_blank=True,
-        help_text=_("Zip/Postal code"))
-    country = serializers.CharField(required=False, allow_blank=True,
-        help_text=_("Country"))
-
-
 class CartItemAPIView(CartMixin, CreateAPIView):
     """
-    Adds an item into a cart
+    Adds an item to the request user cart
 
-    Adds a ``Plan`` into the cart of the ``request.user``.
+    Adds a plan into the cart of the user identified through the HTTP request.
 
-    The cart can later be checked out and paid by an ``Organization``,
-    either through the :ref:`HTML page<pages_cart>`
-    or :ref:`API end point<api_checkout>`.
+    The cart can later be checked out and paid by a billing profile, either
+    through the `HTML checkout page </docs/themes/#workflow_billing_cart>`_
+    or `API end point </docs/api/#createCheckout>`_.
 
     This end point is typically used when a user is presented with a list
     of add-ons that she can subscribes to in one checkout screen. The end-point
@@ -116,19 +68,14 @@ class CartItemAPIView(CartMixin, CreateAPIView):
 
     The end-point accepts a single item or a list of items.
 
-    ``quantity`` is optional. When it is not specified, subsquent checkout
-    screens will provide choices to pay multiple periods in advance
-    When additional ``full_name``, ``email`` and ``sync_on`` are specified,
-    payment can be made by one ``Organization`` for another ``Organization``
-    to be subscribed (see :ref:`GroupBuy orders<group_buy>`).
-
     ``option`` is optional. When it is not specified, subsquent checkout
-    screens will provide choices to pay multiple periods in advance
-    When additional ``full_name`` and ``sync_on`` are specified,
-    payment can be made by one ``Organization`` for another ``Organization``
-    to be subscribed (see :ref:`GroupBuy orders<group_buy>`).
+    screens will provide choices to pay multiple periods in advance.
 
-    **Tags**: billing
+    When additional ``full_name`` and ``sync_on`` are specified,
+    payment can be made by one billing profile for another profile
+    to be subscribed (see :ref:`GroupBuy orders <group_buy>`).
+
+    **Tags**: billing, visitor, cartmodel
 
     **Examples**
 
@@ -139,7 +86,7 @@ class CartItemAPIView(CartMixin, CreateAPIView):
     .. code-block:: json
 
         {
-            "plan": "open-space",
+            "plan": "premium",
             "option": 1
         }
 
@@ -148,7 +95,7 @@ class CartItemAPIView(CartMixin, CreateAPIView):
     .. code-block:: json
 
         {
-            "plan": "open-space",
+            "plan": "premium",
             "option": 1
         }
 
@@ -222,17 +169,17 @@ class CartItemAPIView(CartMixin, CreateAPIView):
 
     def delete(self, request, *args, **kwargs):
         """
-        Removes an item from a cart
+        Removes an item from the request user cart
 
         Removes an item from the ``request.user`` cart.
 
-        **Tags**: billing
+        **Tags**: billing, visitor, cartmodel
 
         **Examples**
 
         .. code-block:: http
 
-            DELETE /api/cart/?plan=open-space HTTP/1.1
+            DELETE /api/cart/?plan=premium HTTP/1.1
         """
         #pylint:disable=unused-argument
         plan = None
@@ -247,13 +194,6 @@ class CartItemAPIView(CartMixin, CreateAPIView):
         return Response(status=status.HTTP_204_NO_CONTENT)
 
 
-class CartItemUploadSerializer(NoModelSerializer):
-
-    created = serializers.CharField()
-    updated = serializers.CharField()
-    failed = serializers.CharField()
-
-
 class CartItemUploadAPIView(CartMixin, GenericAPIView):
     """
     Uploads multiple discount codes into a cart
@@ -266,7 +206,7 @@ class CartItemUploadAPIView(CartMixin, GenericAPIView):
     ``last_name`` and email. The CSV file must not contain a header
     line, only data.
 
-    **Tags**: billing
+    **Tags**: billing, cartmodel
 
     **Examples**
 
@@ -293,12 +233,14 @@ class CartItemUploadAPIView(CartMixin, GenericAPIView):
                 {
                     "first_name": "Joe",
                     "last_name": "Smith",
-                    "email": "joesmith@example.com"
+                    "email": "joesmith@example.com",
+                    "plan": "basic"
                 },
                 {
                     "first_name": "Marie",
                     "last_name": "Johnson",
-                    "email": "mariejohnson@example.com"
+                    "email": "mariejohnson@example.com",
+                    "plan": "basic"
                 }
             ],
             "updated": [],
@@ -351,17 +293,6 @@ class CartItemUploadAPIView(CartMixin, GenericAPIView):
         return Response(response)
 
 
-class RedeemCouponSerializer(NoModelSerializer):
-    """
-    Serializer to redeem a ``Coupon``.
-    """
-
-    code = serializers.CharField(help_text=_("Coupon code to redeem"))
-
-    def create(self, validated_data):
-        return validated_data
-
-
 class CouponRedeemAPIView(GenericAPIView):
     """
     Redeems a discount code
@@ -369,13 +300,13 @@ class CouponRedeemAPIView(GenericAPIView):
     Redeems a ``Coupon`` and applies the discount to the eligible items
     in the cart.
 
-    **Tags**: billing
+    **Tags**: billing, visitor, cartmodel
 
     **Examples**
 
     .. code-block:: http
 
-         POST /api/cart/redeem HTTP/1.1
+         POST /api/cart/redeem/ HTTP/1.1
 
     .. code-block:: json
 
@@ -426,7 +357,7 @@ class CouponRedeemAPIView(GenericAPIView):
 class CheckoutAPIView(InvoicablesMixin, BalanceAndCartMixin,
                       CreateModelMixin, RetrieveAPIView):
     """
-    Retrieves a user cart for checkout
+    Retrieves a cart for checkout
 
     Get a list indexed by plans of items that will be charged
     (`lines`) and options that could be charged instead.
@@ -434,13 +365,17 @@ class CheckoutAPIView(InvoicablesMixin, BalanceAndCartMixin,
     In many subscription businesses, it is possible to buy multiple
     period in advance at a discount. The options reflects that.
 
-    **Tags**: billing
+    The API is typically used within an HTML
+    `checkout page </docs/themes/#workflow_billing_cart>`_
+    as present in the default theme.
+
+    **Tags**: billing, subscriber, cartmodel
 
     **Examples**
 
     .. code-block:: http
 
-        GET /api/billing/xia/checkout HTTP/1.1
+        GET /api/billing/xia/checkout/ HTTP/1.1
 
     responds
 
@@ -493,11 +428,16 @@ class CheckoutAPIView(InvoicablesMixin, BalanceAndCartMixin,
     """
     serializer_class = OrganizationCartSerializer
 
-    @swagger_auto_schema(request_body=CheckoutSerializer, responses={
+    def get_serializer_class(self):
+        if self.request.method.lower() in ('post',):
+            return CheckoutSerializer
+        return super(CheckoutAPIView, self).get_serializer_class()
+
+    @swagger_auto_schema(responses={
         201: OpenAPIResponse("", ChargeSerializer)})
     def post(self, request, *args, **kwargs):
         """
-        Checkouts a user cart
+        Checkouts a cart
 
         Places an order for the subscription items in the cart and creates
         a ``Charge`` on the ``{organization}`` payment card.
@@ -506,17 +446,20 @@ class CheckoutAPIView(InvoicablesMixin, BalanceAndCartMixin,
 
         The cart is manipulated through various API endpoints:
 
-        - `/api/cart/redeem/` applies a coupon code for a potential discount.
-        - `/api/cart/` adds or updates a cart item.
-        - `/api/cart/{plan}` removes a cart item.
+        - `Redeems a discount code </docs/api/#createCouponRedeem>`_ applies a coupon code for a potential discount, and
+        - `Adds an item to the request user cart </docs/api/#createCartItem>`_, `Removes an item from the request user cart </docs/api/#destroyCartItem>`_ to update a cart.
 
-        **Tags**: billing
+        The API is typically used within an HTML
+        `checkout page </docs/themes/#workflow_billing_cart>`_
+        as present in the default theme.
+
+        **Tags**: billing, subscriber, cartmodel
 
         **Examples**
 
         .. code-block:: http
 
-            POST /api/billing/xia/checkout HTTP/1.1
+            POST /api/billing/xia/checkout/ HTTP/1.1
 
         .. code-block:: json
 
@@ -570,7 +513,8 @@ of Xia",
                 opt_index = item['option'] - 1
                 if index >= len(queryset):
                     continue
-                if opt_index >= len(queryset[index]['options']):
+                if (opt_index < 0 or
+                    opt_index >= len(queryset[index]['options'])):
                     continue
                 selected = queryset[index]['options'][opt_index]
                 queryset[index]['lines'].append(selected)
@@ -588,5 +532,6 @@ of Xia",
                 result = ChargeSerializer(charge)
                 return Response(result.data, status=status.HTTP_200_OK)
         except ProcessorError as err:
-            return Response({'detail': err}, status=status.HTTP_400_BAD_REQUEST)
+            return Response({
+                'detail': str(err)}, status=status.HTTP_400_BAD_REQUEST)
         return Response({}, status=status.HTTP_200_OK)
