@@ -60,6 +60,8 @@ Edit the redirect_url and copy/paste the keys into your project settings.py
         }
     }
 """
+#pylint:disable=too-many-lines
+
 from __future__ import unicode_literals
 
 import datetime, logging, re
@@ -123,9 +125,16 @@ class StripeBackend(object):
                     stripe_charge_key, **kwargs)
                 if payment_intent.charges.data:
                     stripe_charge = payment_intent.charges.data[0]
-            except stripe.error.InvalidRequestError:
-                # XXX do more processing here??
-                raise
+            except stripe.error.StripeErrorWithParamCode as err:
+                raise CardError(str(err), err.code, backend_except=err)
+            except stripe.error.AuthenticationError as err:
+                raise ProcessorSetupError(
+                    _("Invalid request on processor for %(organization)s") % {
+                    'organization': broker}, broker, backend_except=err)
+            except stripe.error.StripeError as err:
+                LOGGER.exception(err)
+                raise ProcessorError(str(err), backend_except=err)
+
         return stripe_charge, headers
 
     @staticmethod
@@ -146,6 +155,8 @@ class StripeBackend(object):
                 raise ProcessorSetupError(
                     _("%(organization)s is not connected to a Stripe account."
                     ) % {'organization': broker}, broker)
+            # https://stripe.com/docs/connect/enable-payment-acceptance-guide?\
+            #platform=web#create-a-payment-intent
             kwargs.update({'stripe_account': broker.processor_deposit_key})
         return kwargs
 
@@ -341,7 +352,7 @@ class StripeBackend(object):
             kwargs.update({'destination': provider.processor_deposit_key})
 
         receipt_info = {}
-        if token.startswith('pi_'):
+        if token and token.startswith('pi_'):
             # We are dealing with a PaymentIntent. The Stripe Charge has
             # already been created.
             try:
@@ -352,7 +363,14 @@ class StripeBackend(object):
                     token, kwargs, str(payment_intent))
                 if payment_intent.charges.data:
                     stripe_charge = payment_intent.charges.data[0]
-            except stripe.error.InvalidRequestError as err:
+            except stripe.error.StripeErrorWithParamCode as err:
+                raise CardError(str(err), err.code, backend_except=err)
+            except stripe.error.AuthenticationError as err:
+                raise ProcessorSetupError(
+                    _("Invalid request on processor for %(organization)s") % {
+                    'organization': provider}, provider, backend_except=err)
+            except stripe.error.StripeError as err:
+                LOGGER.exception(err)
                 raise ProcessorError(str(err), backend_except=err)
         else:
             try:
@@ -393,8 +411,7 @@ class StripeBackend(object):
                     " =>\n%s\nstripe_charge=%s",
                     amount, unit, descr, stmt_descr, kwargs,
                     str(payment_intent), str(stripe_charge))
-
-            except stripe.error.CardError as err:
+            except stripe.error.StripeErrorWithParamCode as err:
                 # If the card is declined, Stripe will record a failed
                 # ``Charge`` and raise an exception here. Unfortunately only
                 # the Charge id is present in the CardError exception.
@@ -404,7 +421,8 @@ class StripeBackend(object):
                 raise CardError(str(err), err.code,
                     charge_processor_key=err.json_body['error']['charge'],
                     backend_except=err)
-            except stripe.error.PermissionError as err:
+            except (stripe.error.AuthenticationError,
+                    stripe.error.PermissionError) as err:
                 # It is possible we no longer have access to the connected
                 # account.
                 with transaction.atomic():
@@ -419,8 +437,9 @@ class StripeBackend(object):
                     _("access to %(organization)s Stripe account was denied"\
                       " (access might have been revoked).") % {
                     'organization': provider}, provider, backend_except=err)
-            except stripe.error.IdempotencyError as err:
+            except stripe.error.StripeError as err:
                 LOGGER.exception(err)
+                raise ProcessorError(str(err), backend_except=err)
 
         processor_key = stripe_charge.id
         if created_at is None:
@@ -460,8 +479,16 @@ class StripeBackend(object):
                 idempotency_key=self.generate_idempotent_key(
                     provider, amount, currency, descr),
                 **kwargs)
-        except stripe.error.IdempotencyError as err:
+        except stripe.error.StripeErrorWithParamCode as err:
+            raise CardError(str(err), err.code, backend_except=err)
+        except stripe.error.AuthenticationError as err:
+            raise ProcessorSetupError(
+                _("Invalid request on processor for %(organization)s") % {
+                'organization': provider}, provider, backend_except=err)
+        except stripe.error.StripeError as err:
             LOGGER.exception(err)
+            raise ProcessorError(str(err), backend_except=err)
+
         created_at = utctimestamp_to_datetime(transfer.created)
         return (transfer.id, created_at)
 
@@ -488,6 +515,9 @@ class StripeBackend(object):
                 # We will seamlessly create a new customer on Stripe.
                 LOGGER.warning("Retrieve customer %s on Stripe for %s",
                     subscriber.processor_card_key, subscriber)
+            except stripe.error.StripeError as err:
+                LOGGER.exception(err)
+                raise ProcessorError(str(err), backend_except=err)
 
     def update_bank(self, provider, bank_token):
         """
@@ -499,7 +529,14 @@ class StripeBackend(object):
             # This will only work on "Managed" StripeConnect accounts.
             rcp.external_account = bank_token
             rcp.save()
-        except stripe.error.InvalidRequestError as err:
+        except stripe.error.StripeErrorWithParamCode as err:
+            raise CardError(str(err), err.code, backend_except=err)
+        except stripe.error.AuthenticationError as err:
+            raise ProcessorSetupError(
+                _("Invalid request on processor for %(organization)s") % {
+                'organization': provider}, provider, backend_except=err)
+        except stripe.error.StripeError as err:
+            LOGGER.exception(err)
             raise ProcessorError(str(err), backend_except=err)
 
     def create_or_update_card(self, subscriber, token,
@@ -507,6 +544,7 @@ class StripeBackend(object):
         """
         Create or update a card associated to an subscriber on Stripe.
         """
+        #pylint:disable=too-many-statements
         kwargs = self._prepare_charge_request(broker)
 
         old_card = {}
@@ -527,27 +565,33 @@ class StripeBackend(object):
                 # We will seamlessly create a new customer on Stripe.
                 LOGGER.warning("Retrieve customer %s on Stripe for %s: %s",
                     subscriber.processor_card_key, subscriber, err)
-
-        if p_customer is None:
-            try:
-                p_customer = stripe.Customer.create(
-                    email=subscriber.email, description=subscriber.slug,
-                    idempotency_key=self.generate_idempotent_key(
-                        subscriber, user, broker),
-                    **kwargs)
-            except stripe.error.IdempotencyError as err:
+            except stripe.error.StripeError as err:
                 LOGGER.exception(err)
-            subscriber.processor_card_key = p_customer.id
-            # We rely on ``update_card`` to do the ``save``.
+                raise ProcessorError(str(err), backend_except=err)
 
         if token.startswith('pi_'):
             payment_intent = stripe.PaymentIntent.retrieve(id=token, **kwargs)
             card_token = payment_intent.payment_method
+            if not subscriber.processor_card_key:
+                subscriber.processor_card_key = payment_intent.customer
+                # We rely on caller (``update_card``) to do the ``save``.
         elif token.startswith('seti_'):
             setup_intent = stripe.SetupIntent.retrieve(id=token, **kwargs)
             card_token = setup_intent.payment_method
+            if not subscriber.processor_card_key:
+                subscriber.processor_card_key = setup_intent.customer
+                # We rely on caller (``update_card``) to do the ``save``.
         else:
             try:
+                if p_customer is None:
+                    p_customer = stripe.Customer.create(
+                        email=subscriber.email, description=subscriber.slug,
+                        idempotency_key=self.generate_idempotent_key(
+                            subscriber, user, broker),
+                        **kwargs)
+                    subscriber.processor_card_key = p_customer.id
+                    # We rely on caller (``update_card``) to do the ``save``.
+
                 if self.mode == self.REMOTE:
                     # Implementation note: If we try to create a SetupIntent
                     # on a Connect account, Stripe complains that the card
@@ -573,11 +617,18 @@ class StripeBackend(object):
                             'type': 'card', 'card': {'token': token}},
                         **kwargs)
                     card_token = setup_intent.payment_method
-            except stripe.error.CardError as err:
+            except stripe.error.StripeErrorWithParamCode as err:
                 raise CardError(str(err), err.code, backend_except=err)
+            except stripe.error.AuthenticationError as err:
+                raise ProcessorSetupError(
+                    _("Invalid request on processor for %(organization)s") % {
+                    'organization': broker}, broker, backend_except=err)
+            except stripe.error.StripeError as err:
+                LOGGER.exception(err)
+                raise ProcessorError(str(err), backend_except=err)
 
         try:
-            stripe.Customer.modify(p_customer.id,
+            stripe.Customer.modify(subscriber.processor_card_key,
                 invoice_settings={'default_payment_method': card_token},
                 **kwargs)
             new_card = self._retrieve_card(subscriber, broker=broker)
@@ -585,9 +636,14 @@ class StripeBackend(object):
                 signals.card_updated.send(
                     sender=__name__, organization=subscriber,
                     user=user, old_card=old_card, new_card=new_card)
-        except stripe.error.CardError as err:
+        except stripe.error.StripeErrorWithParamCode as err:
             raise CardError(str(err), err.code, backend_except=err)
-        except stripe.error.InvalidRequestError as err:
+        except stripe.error.AuthenticationError as err:
+            raise ProcessorSetupError(
+                _("Invalid request on processor for %(organization)s") % {
+                'organization': broker}, broker, backend_except=err)
+        except stripe.error.StripeError as err:
+            LOGGER.exception(err)
             raise ProcessorError(str(err), backend_except=err)
 
         return new_card
@@ -597,22 +653,34 @@ class StripeBackend(object):
         Refund a charge on the associated card.
         """
         kwargs = self._prepare_charge_request(charge.broker)
-        refund = stripe.Refund.create(
-            charge=charge.processor_key,
-            amount=amount,
-            refund_application_fee=False,
-            expand=['charge'],
-            **kwargs)
-        LOGGER.debug("refund_charge(charge=%s, amount=%d, broker_amount=%d)"\
-            " => refund=\n%s", charge.processor_key, amount, broker_amount,
-            refund)
-        if broker_amount > 0:
-            application_fee_refund = stripe.ApplicationFee.create_refund(
-                refund.charge.application_fee, amount=broker_amount)
+        try:
+            refund = stripe.Refund.create(
+                charge=charge.processor_key,
+                amount=amount,
+                refund_application_fee=False,
+                expand=['charge'],
+                **kwargs)
             LOGGER.debug(
                 "refund_charge(charge=%s, amount=%d, broker_amount=%d)"\
-                " => application_fee_refund=\n%s", charge.processor_key, amount,
-                broker_amount, application_fee_refund)
+                " => refund=\n%s", charge.processor_key, amount, broker_amount,
+                refund)
+            if broker_amount > 0:
+                application_fee_refund = stripe.ApplicationFee.create_refund(
+                    refund.charge.application_fee, amount=broker_amount)
+                LOGGER.debug(
+                    "refund_charge(charge=%s, amount=%d, broker_amount=%d)"\
+                    " => application_fee_refund=\n%s", charge.processor_key,
+                    amount, broker_amount, application_fee_refund)
+        except stripe.error.StripeErrorWithParamCode as err:
+            raise CardError(str(err), err.code, backend_except=err)
+        except stripe.error.AuthenticationError as err:
+            raise ProcessorSetupError(
+                _("Invalid request on processor for %(organization)s") % {
+                'organization': charge.broker}, charge.broker,
+                backend_except=err)
+        except stripe.error.StripeError as err:
+            LOGGER.exception(err)
+            raise ProcessorError(str(err), backend_except=err)
 
     def get_authorize_url(self, provider, client_id=None, redirect_uri=None):
         if self._is_platform(provider):
@@ -641,7 +709,8 @@ class StripeBackend(object):
         return reverse('saas_deauthorize_processor', args=(provider,))
 
     def get_payment_context(self, provider, processor_card_key,
-                            amount=None, unit=None, broker_fee_amount=0):
+                            amount=None, unit=None, broker_fee_amount=0,
+                            subscriber_email=None, subscriber_slug=None):
         """
         Returns a dictionnary of values that needs to be passed to the browser
         client in order for the processor to create a payment.
@@ -665,62 +734,83 @@ class StripeBackend(object):
                     _("%(organization)s is not connected to a Stripe account."
                     ) % {'organization': provider}, provider)
             kwargs.update({'destination': provider.processor_deposit_key})
-        if processor_card_key is not None:
+
+        try:
+            if processor_card_key is None:
+                p_customer = stripe.Customer.create(
+                    email=subscriber_email, description=subscriber_slug,
+                    idempotency_key=self.generate_idempotent_key(
+                        subscriber_slug, provider),
+                    **kwargs)
+                processor_card_key = p_customer.id
             kwargs.update({'customer': processor_card_key})
 
-        if amount is None:
-            # We are updating a card for later payments.
-            try:
+            if amount is None:
+                # We are updating a card for later payments.
                 setup_intent = stripe.SetupIntent.create(**kwargs)
                 LOGGER.debug("stripe.PaymentIntent.create("\
                     "amount=%d, currency=%s, kwargs=%s) =>\n%s",
                     amount, unit, kwargs, str(setup_intent))
                 context.update({
                     'STRIPE_INTENT_SECRET': setup_intent.client_secret})
-            except stripe.error.InvalidRequestError as err:
-                raise ProcessorSetupError(
-                    _("Invalid request on processor for %(organization)s") % {
-                    'organization': provider}, provider, backend_except=err)
+                if 'stripe_account' in kwargs:
+                    context.update({
+                        'STRIPE_ACCOUNT': kwargs.get('stripe_account')})
 
-        elif amount > 0:
-            if broker_fee_amount:
-                kwargs.update({'application_fee_amount': broker_fee_amount})
-            try:
-                payment_intent = stripe.PaymentIntent.create(
-                    amount=amount, currency=unit,
-                    setup_future_usage='off_session',
-                    **kwargs)
-                LOGGER.debug("stripe.PaymentIntent.create("\
-                    "amount=%d, currency=%s, kwargs=%s) =>\n%s",
-                    amount, unit, kwargs, str(payment_intent))
-                context.update({
-                    'STRIPE_INTENT_SECRET': payment_intent.client_secret})
+            elif amount > 0:
+                if broker_fee_amount:
+                    kwargs.update({'application_fee_amount': broker_fee_amount})
+                try:
+                    payment_intent = stripe.PaymentIntent.create(
+                        amount=amount, currency=unit,
+                        setup_future_usage='off_session',
+                        **kwargs)
+                    LOGGER.debug("stripe.PaymentIntent.create("\
+                        "amount=%d, currency=%s, kwargs=%s) =>\n%s",
+                        amount, unit, kwargs, str(payment_intent))
+                    context.update({
+                        'STRIPE_INTENT_SECRET': payment_intent.client_secret})
+                    if 'stripe_account' in kwargs:
+                        context.update({
+                            'STRIPE_ACCOUNT': kwargs.get('stripe_account')})
 
-            except stripe.error.CardError as err:
-                # If the card is declined, Stripe will record a failed
-                # ``Charge`` and raise an exception here. Unfortunately only
-                # the Charge id is present in the CardError exception.
-                # So instead of generating
-                # an HTTP retrieve and recording a failed charge in our
-                # database, we raise and rollback.
-                raise CardError(str(err), err.code,
-                    charge_processor_key=err.json_body['error']['charge'],
-                    backend_except=err)
-            except stripe.error.PermissionError as err:
-                # It is possible we no longer have access to the connected
-                # account.
-                with transaction.atomic():
-                    # We want this update to be committed even
-                    # if other transactions are unwind on the exception.
-                    provider.processor_pub_key = None
-                    provider.processor_priv_key = None
-                    provider.processor_deposit_key = None
-                    provider.processor_refresh_token = None
-                    provider.save()
-                raise ProcessorSetupError(
-                    _("access to %(organization)s Stripe account was denied"\
-                      " (access might have been revoked).") % {
-                    'organization': provider}, provider, backend_except=err)
+                except stripe.error.StripeErrorWithParamCode as err:
+                    # If the card is declined, Stripe will record a failed
+                    # ``Charge`` and raise an exception here. Unfortunately only
+                    # the Charge id is present in the CardError exception.
+                    # So instead of generating
+                    # an HTTP retrieve and recording a failed charge in our
+                    # database, we raise and rollback.
+                    raise CardError(str(err), err.code,
+                        charge_processor_key=err.json_body['error']['charge'],
+                        backend_except=err)
+                except (stripe.error.AuthenticationError,
+                        stripe.error.PermissionError) as err:
+                    # It is possible we no longer have access to the connected
+                    # account.
+                    with transaction.atomic():
+                        # We want this update to be committed even
+                        # if other transactions are unwind on the exception.
+                        provider.processor_pub_key = None
+                        provider.processor_priv_key = None
+                        provider.processor_deposit_key = None
+                        provider.processor_refresh_token = None
+                        provider.save()
+                    raise ProcessorSetupError(
+                        _("access to %(organization)s Stripe account was"\
+                          " denied (access might have been revoked).") % {
+                        'organization': provider}, provider, backend_except=err)
+
+        except stripe.error.StripeErrorWithParamCode as err:
+            raise CardError(str(err), err.code, backend_except=err)
+        except (stripe.error.AuthenticationError,
+                stripe.error.PermissionError) as err:
+            raise ProcessorSetupError(
+                _("Invalid request on processor for %(organization)s") % {
+                'organization': provider}, provider, backend_except=err)
+        except stripe.error.StripeError as err:
+            LOGGER.exception(err)
+            raise ProcessorError(str(err), backend_except=err)
 
         return context
 
@@ -756,8 +846,17 @@ class StripeBackend(object):
                     context.update({
                         'balance_amount': balance.available[0].amount,
                         'balance_unit': balance.available[0].currency})
+                except stripe.error.StripeErrorWithParamCode as err:
+                    raise CardError(str(err), err.code, backend_except=err)
+                except stripe.error.AuthenticationError as err:
+                    raise ProcessorSetupError(
+                        _("Invalid request on processor for %(organization)s") %
+                        {'organization': provider}, provider,
+                        backend_except=err)
                 except stripe.error.StripeError as err:
+                    LOGGER.exception(err)
                     raise ProcessorError(str(err), backend_except=err)
+
         except ProcessorError:
             # OK here. We don't have a connected Stripe account.
             context.update({
@@ -897,7 +996,15 @@ class StripeBackend(object):
                 transfers = stripe.Payout.list(
                     created={'gt': timestamp}, status='paid',
                     offset=offset, **kwargs)
+        except stripe.error.StripeErrorWithParamCode as err:
+            raise CardError(str(err), err.code, backend_except=err)
+        except stripe.error.AuthenticationError as err:
+            raise ProcessorSetupError(
+                _("Invalid request on processor for %(organization)s") %
+                {'organization': provider}, provider,
+                backend_except=err)
         except stripe.error.StripeError as err:
+            LOGGER.exception(err)
             raise ProcessorError(str(err), backend_except=err)
 
     @staticmethod
